import constants


class Article:

    def __init__(self, article_data):
        self._article_data = article_data

    @property
    def title(self):
        try:
            return self._article_data.find('a', {'class': 'title'}).text.strip()
        except Exception as e:
            return

    @property
    def authors(self):
        try:
            authors = self._article_data.find('span', {'class': 'authors'})
            return [author.text.strip() for author in authors.find_all('a')] if authors is not None else ''
        except Exception as e:
            return

    @property
    def pdf_link(self):
        try:
            return constants.BASE + self._article_data.find('a', {'class': 'pdf-link'})['href']
        except Exception as e:
            return

    @property
    def view_link(self):
        try:
            return constants.BASE + self._article_data.find('a', {'class': 'fulltext'})['href']
        except Exception as e:
            return

    @property
    def publication_title(self):
        try:
            return self._article_data.find('a', {'class': 'publication-title'}).text.strip()
        except Exception as e:
            return

    @property
    def publication_title_link(self):
        try:
            return constants.BASE + self._article_data.find('a', {'class': 'publication-title'})['href']
        except Exception as e:
            return

    @property
    def snippet(self):
        try:
            return self._article_data.find('p', {'class': 'snippet'}).text.strip()
        except Exception as e:
            return

    @property
    def content_type(self):
        try:
            return self._article_data.find('p', {'class': 'content-type'}).text.strip()
        except Exception as e:
            return



from bs4 import BeautifulSoup
from article import Article

import requests
import constants


class Scholar(object):

    def __init__(self):
        self.num_results = 0
        self.csv_link = None

    @staticmethod
    def download(link, filename):
        h = requests.head(link, allow_redirects=True)
        header = h.headers
        content_type = header.get('content-type')
        if 'text' in content_type.lower() or 'html' in content_type.lower():
            return False
        r = requests.get(link, allow_redirects=True)
        print("Downloading: %s" % link)
        open(filename, 'wb').write(r.content)

    def search(self, discipline, query, show_all=False, page=1):
        req = requests.get('%s/search/page/%s?query=%s&facet-discipline=%s&showAll=%s' % (constants.BASE, page, query, discipline, show_all))
        soup = BeautifulSoup(req.content, 'html.parser')
        try:
            nr = soup.find('h1', {'class': 'number-of-search-results-and-search-terms'})
            self.num_results = nr.find('strong').text
            self.num_results = ''.join(c for c in self.num_results if c.isnumeric())
            print("Found %s results" % self.num_results)
            if not int(self.num_results) > 0:
                return False
        except Exception as e:
            print(e.message)
        self.csv_link = constants.BASE + soup.find('a', {'id': 'tool-download'})['href']
        article_list = soup.find('ol', {'id': 'results-list'})
        results = []
        [results.append(Article(a)) for a in article_list.find_all('li')]
        return results



from argparse import ArgumentParser
from prettytable import PrettyTable
from scholar import Scholar
import sys

try:
    from builtins import input
except Exception as e:
    print(e)


def main():
    scholar = Scholar()
    articles = scholar.search(discipline=args.discipline, query=args.query, show_all=args.show_all, page=args.page)
    if not int(scholar.num_results) > 0:
        return False
    row = PrettyTable()
    row.field_names = ["Id", "Title", "Authors"]
    row.align = 'l'
    count = 1
    for article in articles:
        row.add_row([count, article.title, ', '.join(author for author in article.authors)])
        count += 1
    print(row)
    while 1:
        print("You can download a specific article by selecting its id. Run s or save_all to download all files.")
        read = input().lower()
        if read == 's' or read == 'save_all':
            for article in articles:
                scholar.download(article.pdf_link, article.title)
            sys.exit(1)
        try:
            int_val = int(read)
        except (ValueError, TypeError) as error:
            print(error)
            continue
        count = 1
        for article in articles:
            if count == int_val:
                scholar.download(article.pdf_link, article.title)
                sys.exit(1)
            count += 1


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('discipline',
                        help='The study discipline.')
    parser.add_argument('query',
                        help='The search query.',
                        nargs='?')
    parser.add_argument('show_all',
                        default='False',
                        nargs='?',
                        help='Set show_all to False to retrieve preview only content.')
    parser.add_argument('page',
                        default='1',
                        nargs='?',
                        help='The page number.')
    args = parser.parse_args()
    try:
        main()
    except KeyboardInterrupt:
        sys.exit(0)
